package javax.swing.plaf.basic;

import javax.swing.event.ChangeEvent;
import java.beans.PropertyChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.ButtonModel;
import java.awt.event.FocusEvent;
import java.awt.KeyboardFocusManager;
import java.awt.event.MouseEvent;
import java.text.Format;
import javax.swing.text.InternationalFormatter;
import java.util.Map;
import java.text.AttributedCharacterIterator;
import java.text.SimpleDateFormat;
import java.text.DateFormat;
import java.text.ParseException;
import javax.swing.SpinnerDateModel;
import java.awt.event.ActionEvent;
import java.awt.AWTEvent;
import javax.swing.Timer;
import java.awt.event.MouseListener;
import java.awt.event.FocusListener;
import javax.swing.AbstractAction;
import java.awt.Insets;
import javax.swing.InputMap;
import javax.swing.SwingUtilities;
import javax.swing.SpinnerModel;
import java.awt.Container;
import javax.swing.JPanel;
import javax.swing.border.Border;
import javax.swing.border.CompoundBorder;
import javax.swing.plaf.UIResource;
import javax.swing.UIManager;
import java.awt.LayoutManager;
import javax.swing.JButton;
import javax.swing.LookAndFeel;
import javax.swing.JFormattedTextField;
import sun.swing.DefaultLookup;
import java.awt.Component;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import java.awt.Dimension;
import java.beans.PropertyChangeListener;
import javax.swing.JSpinner;
import javax.swing.plaf.SpinnerUI;

public class BasicSpinnerUI extends SpinnerUI
{
    protected JSpinner spinner;
    private BasicSpinnerUI.Handler handler;
    private static final BasicSpinnerUI.ArrowButtonHandler nextButtonHandler;
    private static final BasicSpinnerUI.ArrowButtonHandler previousButtonHandler;
    private PropertyChangeListener propertyChangeListener;
    private static final Dimension zeroSize;
    
    public BasicSpinnerUI() {
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new BasicSpinnerUI();
    }
    
    private void maybeAdd(Component comp, String constraints) {
        if (comp != null) {
            this.spinner.add(comp, constraints);
        }
    }
    
    @Override
    public void installUI(JComponent component) {
        this.spinner = (JSpinner)component;
        this.installDefaults();
        this.installListeners();
        this.maybeAdd(this.createNextButton(), "Next");
        this.maybeAdd(this.createPreviousButton(), "Previous");
        this.maybeAdd(this.createEditor(), "Editor");
        this.updateEnabledState();
        this.installKeyboardActions();
    }
    
    @Override
    public void uninstallUI(JComponent component) {
        this.uninstallDefaults();
        this.uninstallListeners();
        this.spinner = null;
        component.removeAll();
    }
    
    protected void installListeners() {
        this.propertyChangeListener = this.createPropertyChangeListener();
        this.spinner.addPropertyChangeListener(this.propertyChangeListener);
        if (DefaultLookup.getBoolean(this.spinner, this, "Spinner.disableOnBoundaryValues", false)) {
            this.spinner.addChangeListener(this.getHandler());
        }
        JComponent editor = this.spinner.getEditor();
        if (editor != null && editor instanceof JSpinner.DefaultEditor) {
            JFormattedTextField textField = ((JSpinner.DefaultEditor)editor).getTextField();
            if (textField != null) {
                textField.addFocusListener(BasicSpinnerUI.nextButtonHandler);
                textField.addFocusListener(BasicSpinnerUI.previousButtonHandler);
            }
        }
    }
    
    protected void uninstallListeners() {
        this.spinner.removePropertyChangeListener(this.propertyChangeListener);
        this.spinner.removeChangeListener(this.handler);
        JComponent editor = this.spinner.getEditor();
        this.removeEditorBorderListener(editor);
        if ((editor instanceof JSpinner.DefaultEditor)) {
            JFormattedTextField textField = ((JSpinner.DefaultEditor)editor).getTextField();
            if (textField != null) {
                textField.removeFocusListener(BasicSpinnerUI.nextButtonHandler);
                textField.removeFocusListener(BasicSpinnerUI.previousButtonHandler);
            }
        }
        this.propertyChangeListener = null;
        this.handler = null;
    }
    
    protected void installDefaults() {
        this.spinner.setLayout(this.createLayout());
        LookAndFeel.installBorder(this.spinner, "Spinner.border");
        LookAndFeel.installColorsAndFont(this.spinner, "Spinner.background", "Spinner.foreground", "Spinner.font");
        LookAndFeel.installProperty(this.spinner, "opaque", Boolean.TRUE);
    }
    
    protected void uninstallDefaults() {
        this.spinner.setLayout(null);
    }
    
    private BasicSpinnerUI.Handler getHandler() {
        if (this.handler == null) {
            this.handler = new BasicSpinnerUI.Handler();
        }
        return this.handler;
    }
    
    protected void installNextButtonListeners(Component component) {
        this.installButtonListeners(component, BasicSpinnerUI.nextButtonHandler);
    }
    
    protected void installPreviousButtonListeners(Component component) {
        this.installButtonListeners(component, BasicSpinnerUI.previousButtonHandler);
    }
    
    private void installButtonListeners(Component component, BasicSpinnerUI.ArrowButtonHandler arrowButtonHandler) {
        if ((component instanceof JButton)) {
            ((JButton)component).addActionListener(arrowButtonHandler);
        }
        component.addMouseListener(arrowButtonHandler);
    }
    
    protected LayoutManager createLayout() {
        return this.getHandler();
    }
    
    protected PropertyChangeListener createPropertyChangeListener() {
        return this.getHandler();
    }
    
    protected Component createPreviousButton() {
        Component arrowButton = this.createArrowButton(5);
        arrowButton.setName("Spinner.previousButton");
        this.installPreviousButtonListeners(arrowButton);
        return arrowButton;
    }
    
    protected Component createNextButton() {
        Component arrowButton = this.createArrowButton(1);
        arrowButton.setName("Spinner.nextButton");
        this.installNextButtonListeners(arrowButton);
        return arrowButton;
    }
    
    private Component createArrowButton(int n) {
        BasicArrowButton basicArrowButton = new BasicArrowButton(n);
        Border border = UIManager.getBorder("Spinner.arrowButtonBorder");
        if ((border instanceof UIResource)) {
            basicArrowButton.setBorder(new CompoundBorder(border, null));
        }
        else {
            basicArrowButton.setBorder(border);
        }
        basicArrowButton.setInheritsPopupMenu(true);
        return basicArrowButton;
    }
    
    protected JComponent createEditor() {
        JComponent editor = this.spinner.getEditor();
        this.maybeRemoveEditorBorder(editor);
        this.installEditorBorderListener(editor);
        editor.setInheritsPopupMenu(true);
        this.updateEditorAlignment(editor);
        return editor;
    }
    
    protected void replaceEditor(JComponent comp, JComponent comp2) {
        this.spinner.remove(comp);
        this.maybeRemoveEditorBorder(comp2);
        this.installEditorBorderListener(comp2);
        comp2.setInheritsPopupMenu(true);
        this.spinner.add(comp2, "Editor");
    }
    
    private void updateEditorAlignment(JComponent component) {
        if ((component instanceof JSpinner.DefaultEditor)) {
            ((JSpinner.DefaultEditor)component).getTextField().setHorizontalAlignment(UIManager.getInt("Spinner.editorAlignment"));
        }
    }
    
    private void maybeRemoveEditorBorder(JComponent component) {
        if (!UIManager.getBoolean("Spinner.editorBorderPainted")) {
            if (component instanceof JPanel && component.getBorder() == null && component.getComponentCount() > 0) {
                component = (JComponent)component.getComponent(0);
            }
            if (component != null && component.getBorder() instanceof UIResource) {
                component.setBorder(null);
            }
        }
    }
    
    private void installEditorBorderListener(JComponent component) {
        if (!UIManager.getBoolean("Spinner.editorBorderPainted")) {
            if (component instanceof JPanel && component.getBorder() == null && component.getComponentCount() > 0) {
                component = (JComponent)component.getComponent(0);
            }
            if (component != null && (component.getBorder() == null || component.getBorder() instanceof UIResource)) {
                component.addPropertyChangeListener(this.getHandler());
            }
        }
    }
    
    private void removeEditorBorderListener(JComponent component) {
        if (!UIManager.getBoolean("Spinner.editorBorderPainted")) {
            if (component instanceof JPanel && component.getComponentCount() > 0) {
                component = (JComponent)component.getComponent(0);
            }
            if (component != null) {
                component.removePropertyChangeListener(this.getHandler());
            }
        }
    }
    
    private void updateEnabledState() {
        this.updateEnabledState(this.spinner, this.spinner.isEnabled());
    }
    
    private void updateEnabledState(Container container, boolean b) {
        for (int i = container.getComponentCount() - 1; i >= 0; --i) {
            Component component = container.getComponent(i);
            if (DefaultLookup.getBoolean(this.spinner, this, "Spinner.disableOnBoundaryValues", false)) {
                SpinnerModel model = this.spinner.getModel();
                if (component.getName() == "Spinner.nextButton" && model.getNextValue() == null) {
                    component.setEnabled(false);
                }
                else if (component.getName() == "Spinner.previousButton" && model.getPreviousValue() == null) {
                    component.setEnabled(false);
                }
                else {
                    component.setEnabled(b);
                }
            }
            else {
                component.setEnabled(b);
            }
            if ((component instanceof Container)) {
                this.updateEnabledState((Container)component, b);
            }
        }
    }
    
    protected void installKeyboardActions() {
        SwingUtilities.replaceUIInputMap(this.spinner, 1, this.getInputMap(1));
        LazyActionMap.installLazyActionMap(this.spinner, BasicSpinnerUI.class, "Spinner.actionMap");
    }
    
    private InputMap getInputMap(int n) {
        if (n == 1) {
            return (InputMap)DefaultLookup.get(this.spinner, this, "Spinner.ancestorInputMap");
        }
        return null;
    }
    
    static void loadActionMap(LazyActionMap lazyActionMap) {
        lazyActionMap.put("increment", BasicSpinnerUI.nextButtonHandler);
        lazyActionMap.put("decrement", BasicSpinnerUI.previousButtonHandler);
    }
    
    @Override
    public int getBaseline(JComponent c, int n, int n2) {
        super.getBaseline(c, n, n2);
        JComponent editor = this.spinner.getEditor();
        Insets insets = this.spinner.getInsets();
        n = n - insets.left - insets.right;
        n2 = n2 - insets.top - insets.bottom;
        if (n >= 0 && n2 >= 0) {
            int baseline = editor.getBaseline(n, n2);
            if (baseline >= 0) {
                return insets.top + baseline;
            }
        }
        return -1;
    }
    
    @Override
    public Component.BaselineResizeBehavior getBaselineResizeBehavior(JComponent c) {
        super.getBaselineResizeBehavior(c);
        return this.spinner.getEditor().getBaselineResizeBehavior();
    }
    
    static void access$200(BasicSpinnerUI basicSpinnerUI) {
        basicSpinnerUI.updateEnabledState();
    }
    
    static void access$500(BasicSpinnerUI basicSpinnerUI, JComponent component) {
        basicSpinnerUI.maybeRemoveEditorBorder(component);
    }
    
    static {
        nextButtonHandler = new BasicSpinnerUI.ArrowButtonHandler("increment", true);
        previousButtonHandler = new BasicSpinnerUI.ArrowButtonHandler("decrement", false);
        zeroSize = new Dimension(0, 0);
    }
    
    private static class ArrowButtonHandler extends AbstractAction implements FocusListener, MouseListener, UIResource
    {
        final Timer autoRepeatTimer;
        final boolean isNext;
        JSpinner spinner;
        JButton arrowButton;
        
        ArrowButtonHandler(String name, boolean isNext) {
            super(name);
            this.spinner = null;
            this.arrowButton = null;
            this.isNext = isNext;
            (this.autoRepeatTimer = new Timer(60, this)).setInitialDelay(300);
        }
        
        private JSpinner eventToSpinner(AWTEvent awtEvent) {
            Object o;
            for (o = awtEvent.getSource(); o instanceof Component && !(o instanceof JSpinner); o = ((Component)o).getParent()) {}
            return (o instanceof JSpinner) ? ((JSpinner)o) : null;
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JSpinner spinner = this.spinner;
            if (!(actionEvent.getSource() instanceof Timer)) {
                spinner = this.eventToSpinner(actionEvent);
                if ((actionEvent.getSource() instanceof JButton)) {
                    this.arrowButton = (JButton)actionEvent.getSource();
                }
            }
            else if (this.arrowButton != null && !this.arrowButton.getModel().isPressed() && this.autoRepeatTimer.isRunning()) {
                this.autoRepeatTimer.stop();
                spinner = null;
                this.arrowButton = null;
            }
            if (spinner != null) {
                try {
                    int calendarField = this.getCalendarField(spinner);
                    spinner.commitEdit();
                    if (calendarField != -1) {
                        ((SpinnerDateModel)spinner.getModel()).setCalendarField(calendarField);
                    }
                    Object value = this.isNext ? spinner.getNextValue() : spinner.getPreviousValue();
                    if (value != null) {
                        spinner.setValue(value);
                        this.select(spinner);
                    }
                }
                catch (IllegalArgumentException ex) {
                    UIManager.getLookAndFeel().provideErrorFeedback(spinner);
                }
                catch (ParseException ex2) {
                    UIManager.getLookAndFeel().provideErrorFeedback(spinner);
                }
            }
        }
        
        private void select(JSpinner spinner) {
            JComponent editor = spinner.getEditor();
            if ((editor instanceof JSpinner.DateEditor)) {
                JSpinner.DateEditor dateEditor = (JSpinner.DateEditor)editor;
                JFormattedTextField textField = dateEditor.getTextField();
                SimpleDateFormat format = dateEditor.getFormat();
                Object value;
                if (format != null && (value = spinner.getValue()) != null) {
                    DateFormat.Field ofCalendarField = DateFormat.Field.ofCalendarField(dateEditor.getModel().getCalendarField());
                    if (ofCalendarField != null) {
                        try {
                            AttributedCharacterIterator formatToCharacterIterator = format.formatToCharacterIterator(value);
                            if (!this.select(textField, formatToCharacterIterator, ofCalendarField) && ofCalendarField == DateFormat.Field.HOUR0) {
                                this.select(textField, formatToCharacterIterator, DateFormat.Field.HOUR1);
                            }
                        }
                        catch (IllegalArgumentException ex) {}
                    }
                }
            }
        }
        
        private boolean select(JFormattedTextField formattedTextField, AttributedCharacterIterator attributedCharacterIterator, DateFormat.Field field) {
            int length = formattedTextField.getDocument().getLength();
            attributedCharacterIterator.first();
            do {
                Map<AttributedCharacterIterator.Attribute, Object> attributes = attributedCharacterIterator.getAttributes();
                if (attributes != null && attributes.containsKey(field)) {
                    int runStart = attributedCharacterIterator.getRunStart(field);
                    int runLimit = attributedCharacterIterator.getRunLimit(field);
                    if (runStart != -1 && runLimit != -1 && runStart <= length && runLimit <= length) {
                        formattedTextField.select(runStart, runLimit);
                    }
                    return true;
                }
                else {
                    continue;
                }
            } while (attributedCharacterIterator.next() != '\uffff');
            return false;
        }
        
        private int getCalendarField(JSpinner spinner) {
            JComponent editor = spinner.getEditor();
            if ((editor instanceof JSpinner.DateEditor)) {
                JFormattedTextField textField = ((JSpinner.DateEditor)editor).getTextField();
                int selectionStart = textField.getSelectionStart();
                JFormattedTextField.AbstractFormatter formatter = textField.getFormatter();
                if ((formatter instanceof InternationalFormatter)) {
                    Format.Field[] fields = ((InternationalFormatter)formatter).getFields(selectionStart);
                    for (int i = 0; i < fields.length; ++i) {
                        if ((fields[i] instanceof DateFormat.Field)) {
                            int calendarField;
                            if (fields[i] == DateFormat.Field.HOUR1) {
                                calendarField = 10;
                            }
                            else {
                                calendarField = ((DateFormat.Field)fields[i]).getCalendarField();
                            }
                            if (calendarField != -1) {
                                return calendarField;
                            }
                        }
                    }
                }
            }
            return -1;
        }
        
        @Override
        public void mousePressed(MouseEvent anEvent) {
            if (SwingUtilities.isLeftMouseButton(anEvent) && anEvent.getComponent().isEnabled()) {
                this.spinner = this.eventToSpinner(anEvent);
                this.autoRepeatTimer.start();
                this.focusSpinnerIfNecessary();
            }
        }
        
        @Override
        public void mouseReleased(MouseEvent mouseEvent) {
            this.autoRepeatTimer.stop();
            this.arrowButton = null;
            this.spinner = null;
        }
        
        @Override
        public void mouseClicked(MouseEvent mouseEvent) {
        }
        
        @Override
        public void mouseEntered(MouseEvent mouseEvent) {
            if (this.spinner != null && !this.autoRepeatTimer.isRunning() && this.spinner == this.eventToSpinner(mouseEvent)) {
                this.autoRepeatTimer.start();
            }
        }
        
        @Override
        public void mouseExited(MouseEvent mouseEvent) {
            if (this.autoRepeatTimer.isRunning()) {
                this.autoRepeatTimer.stop();
            }
        }
        
        private void focusSpinnerIfNecessary() {
            Component focusOwner = KeyboardFocusManager.getCurrentKeyboardFocusManager().getFocusOwner();
            if (this.spinner.isRequestFocusEnabled() && (focusOwner == null || !SwingUtilities.isDescendingFrom(focusOwner, this.spinner))) {
                JSpinner spinner = this.spinner;
                if (!spinner.isFocusCycleRoot()) {
                    spinner = spinner.getFocusCycleRootAncestor();
                }
                if (spinner != null) {
                    Component componentAfter = spinner.getFocusTraversalPolicy().getComponentAfter(spinner, this.spinner);
                    if (componentAfter != null && SwingUtilities.isDescendingFrom(componentAfter, this.spinner)) {
                        componentAfter.requestFocus();
                    }
                }
            }
        }
        
        @Override
        public void focusGained(FocusEvent focusEvent) {
        }
        
        @Override
        public void focusLost(FocusEvent focusEvent) {
            if (this.spinner == this.eventToSpinner(focusEvent)) {
                if (this.autoRepeatTimer.isRunning()) {
                    this.autoRepeatTimer.stop();
                }
                this.spinner = null;
                if (this.arrowButton != null) {
                    ButtonModel model = this.arrowButton.getModel();
                    model.setPressed(false);
                    model.setArmed(false);
                    this.arrowButton = null;
                }
            }
        }
    }
    
    private static class Handler implements LayoutManager, PropertyChangeListener, ChangeListener
    {
        private Component nextButton;
        private Component previousButton;
        private Component editor;
        
        private Handler() {
            this.nextButton = null;
            this.previousButton = null;
            this.editor = null;
        }
        
        @Override
        public void addLayoutComponent(String anObject, Component nextButton) {
            if ("Next".equals(anObject)) {
                this.nextButton = nextButton;
            }
            else if ("Previous".equals(anObject)) {
                this.previousButton = nextButton;
            }
            else if ("Editor".equals(anObject)) {
                this.editor = nextButton;
            }
        }
        
        @Override
        public void removeLayoutComponent(Component component) {
            if (component == this.nextButton) {
                this.nextButton = null;
            }
            else if (component == this.previousButton) {
                this.previousButton = null;
            }
            else if (component == this.editor) {
                this.editor = null;
            }
        }
        
        private Dimension preferredSize(Component component) {
            return (component == null) ? BasicSpinnerUI.zeroSize : component.getPreferredSize();
        }
        
        @Override
        public Dimension preferredLayoutSize(Container container) {
            Dimension preferredSize = this.preferredSize(this.nextButton);
            Dimension preferredSize2 = this.preferredSize(this.previousButton);
            Dimension preferredSize3 = this.preferredSize(this.editor);
            preferredSize3.height = (preferredSize3.height + 1) / 2 * 2;
            Dimension dimension = new Dimension(preferredSize3.width, preferredSize3.height);
            dimension.width += Math.max(preferredSize.width, preferredSize2.width);
            Insets insets = container.getInsets();
            dimension.width += insets.left + insets.right;
            dimension.height += insets.top + insets.bottom;
            return dimension;
        }
        
        @Override
        public Dimension minimumLayoutSize(Container container) {
            return this.preferredLayoutSize(container);
        }
        
        private void setBounds(Component component, int x, int y, int width, int height) {
            if (component != null) {
                component.setBounds(x, y, width, height);
            }
        }
        
        @Override
        public void layoutContainer(Container container) {
            int width = container.getWidth();
            int height = container.getHeight();
            Insets insets = container.getInsets();
            if (this.nextButton == null && this.previousButton == null) {
                this.setBounds(this.editor, insets.left, insets.top, width - insets.left - insets.right, height - insets.top - insets.bottom);
                return;
            }
            int max = Math.max(this.preferredSize(this.nextButton).width, this.preferredSize(this.previousButton).width);
            int n = height - (insets.top + insets.bottom);
            Insets insets2 = UIManager.getInsets("Spinner.arrowButtonInsets");
            if (insets2 == null) {
                insets2 = insets;
            }
            int left;
            int left2;
            int n2;
            if (container.getComponentOrientation().isLeftToRight()) {
                left2 = insets.left;
                n2 = width - insets.left - max - insets2.right;
                left = width - max - insets2.right;
            }
            else {
                left = insets2.left;
                left2 = left + max;
                n2 = width - insets2.left - max - insets.right;
            }
            int top = insets2.top;
            int n3 = height / 2 + height % 2 - top;
            int n4 = insets2.top + n3;
            int n5 = height - n4 - insets2.bottom;
            this.setBounds(this.editor, left2, insets.top, n2, n);
            this.setBounds(this.nextButton, left, top, max, n3);
            this.setBounds(this.previousButton, left, n4, max, n5);
            return;
        }
        
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            String propertyName = propertyChangeEvent.getPropertyName();
            if ((propertyChangeEvent.getSource() instanceof JSpinner)) {
                JSpinner spinner = (JSpinner)(JSpinner)propertyChangeEvent.getSource();
                SpinnerUI ui2 = spinner.getUI();
                if ((ui2 instanceof BasicSpinnerUI)) {
                    BasicSpinnerUI basicSpinnerUI = (BasicSpinnerUI)ui2;
                    if ("editor".equals(propertyName)) {
                        JComponent component2 = (JComponent)propertyChangeEvent.getOldValue();
                        JComponent component3 = (JComponent)propertyChangeEvent.getNewValue();
                        basicSpinnerUI.replaceEditor(component2, component3);
                        BasicSpinnerUI.access$200(basicSpinnerUI);
                        if ((component2 instanceof JSpinner.DefaultEditor)) {
                            JFormattedTextField textField2 = ((JSpinner.DefaultEditor)component2).getTextField();
                            if (textField2 != null) {
                                textField2.removeFocusListener(BasicSpinnerUI.nextButtonHandler);
                                textField2.removeFocusListener(BasicSpinnerUI.previousButtonHandler);
                            }
                        }
                        if ((component3 instanceof JSpinner.DefaultEditor)) {
                            JFormattedTextField textField3 = ((JSpinner.DefaultEditor)component3).getTextField();
                            if (textField3 != null) {
                                if ((textField3.getFont() instanceof UIResource)) {
                                    textField3.setFont(spinner.getFont());
                                }
                                textField3.addFocusListener(BasicSpinnerUI.nextButtonHandler);
                                textField3.addFocusListener(BasicSpinnerUI.previousButtonHandler);
                            }
                        }
                    }
                    else if ("enabled".equals(propertyName) || "model".equals(propertyName)) {
                        BasicSpinnerUI.access$200(basicSpinnerUI);
                    }
                    else if ("font".equals(propertyName)) {
                        JComponent editor = spinner.getEditor();
                        if (editor != null && editor instanceof JSpinner.DefaultEditor) {
                            JFormattedTextField textField = ((JSpinner.DefaultEditor)editor).getTextField();
                            if (textField != null && textField.getFont() instanceof UIResource) {
                                textField.setFont(spinner.getFont());
                            }
                        }
                    }
                    else if ("ToolTipText".equals(propertyName)) {
                        this.updateToolTipTextForChildren(spinner);
                    }
                }
            }
            else if ((propertyChangeEvent.getSource() instanceof JComponent)) {
                JComponent component = (JComponent)propertyChangeEvent.getSource();
                if (component.getParent() instanceof JPanel && component.getParent().getParent() instanceof JSpinner && "border".equals(propertyName)) {
                    SpinnerUI ui = ((JSpinner)component.getParent().getParent()).getUI();
                    if ((ui instanceof BasicSpinnerUI)) {
                        BasicSpinnerUI.access$500((BasicSpinnerUI)ui, component);
                    }
                }
            }
        }
        
        private void updateToolTipTextForChildren(JComponent component) {
            String toolTipText = component.getToolTipText();
            Component[] components = component.getComponents();
            for (int i = 0; i < components.length; ++i) {
                if ((components[i] instanceof JSpinner.DefaultEditor)) {
                    JFormattedTextField textField = ((JSpinner.DefaultEditor)components[i]).getTextField();
                    if (textField != null) {
                        textField.setToolTipText(toolTipText);
                    }
                }
                else if ((components[i] instanceof JComponent)) {
                    ((JComponent)components[i]).setToolTipText(component.getToolTipText());
                }
            }
        }
        
        @Override
        public void stateChanged(ChangeEvent changeEvent) {
            if ((changeEvent.getSource() instanceof JSpinner)) {
                JSpinner c = (JSpinner)changeEvent.getSource();
                SpinnerUI ui = c.getUI();
                if (DefaultLookup.getBoolean(c, ui, "Spinner.disableOnBoundaryValues", false) && ui instanceof BasicSpinnerUI) {
                    BasicSpinnerUI.access$200((BasicSpinnerUI)ui);
                }
            }
        }
    }
}
